#####################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../standard.4gs/colspecs.4gl  1.2  Delta: 1/24/91
#####################################################################


# static initialization flag
define
    prep_flag smallint

#####################################################################
function col_specs(tab_name, col_name)
# returns column type string and length
#####################################################################
#
    define
        tab_name    char(18),
        col_name    char(18),
        query_str   char(200),
        c_length    smallint,
        c_type      smallint,
        result_type char(30),
        result_len  smallint

    # Trap fatal errors
    whenever error call error_handler

    # initialize
    if prep_flag = 123
    then else
        # turn initialization flag off
        let prep_flag = 123

        # build the select to get the column data
        let query_str =
            "select syscolumns.coltype, syscolumns.collength ",
            "from syscolumns, systables ",
            "where syscolumns.colname = ? and ",
                "systables.tabname = ? and ",
                "syscolumns.tabid = systables.tabid"

        # prepare and declare the selection cursor
        prepare col_query from query_str
        declare col_cur cursor for col_query
    end if
    # end one time initialization

    # open the cursor and fetch the result
    whenever error continue
    open col_cur using col_name, tab_name
    fetch col_cur into c_type, c_length

    # check status for no result
    if status
      then return "", 0
    end if

    # Close the cursor after status is checked
    close col_cur
    whenever error call error_handler

    # get the type string and length
    call col_type(c_type, c_length) returning result_type, result_len

    # return the length and type
    return result_type, result_len

end function
# end col_specs()


#####################################################################
function col_type(coltype, collength)
# returning a 30 character text string representing the column type
# and the display length of the column.
#####################################################################
#  This function is passed the (Informix) coded coltype and collength
#  and returns the column type as a string.
#
#  The coding scheme follows:
#     coltype:       collength:
#     0 = char(n)       n
#     1 = smallint      2
#     2 = integer       4
#     3 = float         sizeof(double)
#     4 = smallfloat    sizeof(float)
#     5 = decimal       (see note 2)
#     6 = serial        4
#     7 = date          4
#     8 = money         (see note 2)
#
#  NOTE 1:  Add 256 to the coltype value if the column
#           is of "NOT NULL" type also.
#  NOTE 2:  The collength argument is used when coltype is char, and
#           if the column is of type decimal or money.  In the decimal
#           or money situation, the length of the column is determined
#           using the following formula:
#
#             for m and n in decimal(m,n)
#               m = integer portion of: (collength/256)
#               n = collength - (m * 256)
#               ---- or, using "C": -----
#               m = (collength >> 8) & 0xff
#               n = collength & 0xff
#
#           If the precision (n) is 255, then it is a "float" type.
#
#  Examples:
#    col_type(2,4)   returns "integer", 4
#    col_type(262,4) returns "serial not null", 4
#    col_type(264,4098) returns "money (16,2) not null", 16
#    col_type(264,2815) returns "money (10) not null", 10
#
    define
        coltype smallint,    # coded type
        collength smallint,  # coded length
        not_null smallint,   # boolean: is this a "not null" type?
        m smallint,          # storage for the total length
        n smallint,          # storage for the decimal precision
        scratch char(30)     # string to return

    let not_null = false

    # check for not null
    if coltype >= 256
    then
        let not_null = true
        let coltype = coltype - 256
    end if

    # calculate the total and precision
    # if the type is decimal or money
    if coltype = 5 or coltype = 8
    then
        let m = collength / 256
        let n = collength - (256 * m)
    end if

    case
    when coltype = 0
        let scratch = "char(", collength using "<<<<<<", ")"
        let m = collength
    when coltype = 1
        let scratch = "smallint"
        let m = 5
    when coltype = 2
        let scratch = "integer"
        let m = 10
    when coltype = 3
        let scratch = "float"
        let m = 16
    when coltype = 4
        let scratch = "smallfloat"
        let m = 8
    when coltype = 5
        if n = 255
        then
            let scratch = "decimal(", m using "<<<", ")"  # float
        else
            let scratch = "decimal(", m using "<<<",
                ",", n using "<<<&", ")"
        end if
        # Bump up field length for the decimal (.) character
        let n = n + 1
    when coltype = 6
        let scratch = "serial"
        let m = 10
    when coltype = 7
        let scratch = "date"
        let m = 8
    when coltype = 8
        if n = 255
        then
            let scratch = "money(", m using "<<<", ")"  # float
        else
            let scratch = "money(", m using "<<<",
                ",", n using "<<<&", ")"
        end if
        # Bump up field length for '$' and '.' characters
        let n = n + 2
    otherwise
        let scratch = ""
        let m = 0
    end case

    # add the "not null" string
    if not_null
    then
        let scratch = scratch clipped, " not null"
    end if

    # return the column type string and length
    let n = length(scratch)
	if n < 1 then return "", 0 end if
    return scratch[1,n], m

end function
# col_type()
