######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../standard.4gs/fg_ack.4gl  1.1  Delta: 11/23/90
######################################################################

# This 4GL module contains the functions necessary for the
# user to update or query the program acknowledgements

globals "globals.4gl"

######################################################################
function fg_ack()
######################################################################
# This function calls the picker on all acknowledgements.  If the
# user selects zoom (control-z) in the picker, it opens the
# editor & writes the new info out to the acknowledgement table.
#
    define
        a_mod char(8),    # program module
        a_prog char(8),   # program name
        quote char(1),    # storage for the quote character (")
        sel_txt char(150),# storage for the selection text
        n smallint        # generic counter

    # Trap fatal errors
    whenever error call error_handler

    let quote = "\""
    # Parse the module.program from progid.
    let a_mod = ""
    # Search for the period delimiter
    for n = 1 to 9
        if progid[n] = "."
        then
            let a_mod = progid[1,n-1]
            let a_prog = progid[n+1,17]
            exit for
        end if
    end for
    # Delimiter not found
    if a_prog is null then let a_prog = progid end if

    # Select ack. text
    let sel_txt = "select ack_text, line_no from stxacknd ",
      "where ack_module  = ", quote, a_mod clipped, quote,
      "  and ack_program = ", quote, a_prog clipped, quote,
      " order by line_no"
    let scratch = sel_txt
    call textsel() returning n

    # The following lines constitute the FOURGEN copyright notice.
    # The removal or alteration of these lines is a violation of
    # federal copyright law.
    call textput("========================================")
    call textput("Written using the FOURGEN code generator")
    call textput("Author:  Loren West")
    call textput("Copyright (c) 1989, 1990")
    call textput("FourGen Software Technologies, Inc.")
    call textput("Edmonds, Washington  USA")
    call textput("========================================")

    # Call the picker first
    call texthelp("lib_scr","ack",1)
    call textzoom()
    call textview("Software Acknowledgements")
    if scratch[1,4] = "zoom"
    then
        # Re-select to remove the hardcoded copyright lines
        call textget() returning n  # reset the edit array
        let scratch = sel_txt
        call textsel() returning n

        # Enter text editor
        call texthelp("lib_scr","ack",2)
        call textdefault("table: stxacknd")
        if textedit("Acknowledgements",60)
        then
            # Delete old text
            whenever error continue
            delete from stxacknd where ack_module = a_mod and
              ack_program = a_prog
            whenever error call error_handler
            if sqlca.sqlcode < 0
            then
                call lib_error("lib_scr","ack",1,"can't delete")
                return
            end if

            # Insert new text
            let n = 1
            while textget()
                insert into stxacknd
                  values(a_mod, a_prog, n, scratch)
                let n = n + 1
            end while
        end if
    end if

end function
# fg_ack()
