######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../standard.4gs/fg_file.4gl  1.1  Delta: 11/23/90
######################################################################

globals "globals.4gl"

define
    last_cmd char(512),
    uniq_no integer

######################################################################
function fg_file(cmd_line)
# returning true if there is an element in scratch to return
# returning false if there is no element in scratch to return
######################################################################
# This function is used to read an OS command (cmd_line) into memory.
# It is designed to be used like this:
#   while fg_file("os_command")
#       let array[n] = scratch
#       let n = n + 1
#   end while
#
    define
        cmd_line char(512),   # command line to run
        load_scr char(14),    # load script filename
        load_file char(14),   # load file filename
        dbname char(18),      # current database name
        line_no integer,
        quote char(1),
        n smallint

    # Trap fatal errors
    whenever error call error_handler

    # Is this the first call with this cmd?
    if last_cmd = cmd_line
    then else
        # Cleanup any previous data in the file
        if last_cmd is not null
          then delete from stxfiler where uniq_num = uniq_no end if

        # Generate a unique number
        # (this will build the database tables if needed)
        let uniq_no = al_uniq()
        let load_scr  = uniq_no using "dbld<<<<<<<<<<"
        let load_file = uniq_no using "tmp_<<<<<<<<<<"
        let quote = "\""
        let dbname = get_scrlib("dbname")
        if dbname is null then return false end if

        # output the dbload format to the dbld file.
        start report fg_oneline to load_scr
        let scratch = "file ", quote, load_file clipped, quote, 
          " delimiter ", quote, quote, " 1;"
        output to report fg_oneline(scratch[1,200])
        let scratch = "insert into stxfiler values(",quote,
          uniq_no using "<<<<<<<<<<", quote, ",0,f01);"
        output to report fg_oneline(scratch[1,200])
        finish report fg_oneline
    
        # Build the temporary file to be loaded
        let scratch = "trap '' 2;(",
          cmd_line clipped, ") > ",load_file clipped, " 2>&1"
        run scratch

        # load the file into the database table: stxfiler
        let scratch = "trap '' 2; dbload -d ", dbname clipped,
          " -c ", load_scr clipped, " -l errfile >/dev/null 2>&1 ",
          "; rm -f errfile ", load_scr clipped, " ", load_file
        run scratch
    
        # build a cursor on stxfiler
        let scratch = "select line_no, line_text from stxfiler ",
          "where uniq_num = ", uniq_no , " order by line_no"
        prepare file_specs from scratch
        declare file_curs cursor for file_specs
        open file_curs
        let last_cmd = cmd_line
    end if

    # Read the line from the cursor
    fetch file_curs into line_no, scratch

    # Delete the data on the last (notfound) fetch
    if sqlca.sqlcode = notfound
    then
        close file_curs

        # Only delete if we're not using transactions.  If we're using
        # database transactions, this following delete will render
        # subsequent calls to fg_file() inoperable because the dbload
        # program won't add rows to a table that has a delete lock
        # held (for some reason).
        if is_trx() = false
          then delete from stxfiler where uniq_num = uniq_no
        end if

        let last_cmd = null
        return false
    end if
    return true
end function
# fg_file()
