#####################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../standard.4gs/lib_text.4gl  1.1  Delta: 11/23/90
#####################################################################

globals "globals.4gl"

define
    heading char(80),          # window heading
    arr_cur smallint,          # current array element
    arr_cnt smallint,          # number of active array elements
    reset_cnt smallint,        # should the array counter be reset?
    is_zoom smallint,          # display the (Zoom) message?
    is_open char(1),           # is the window currently open? (Y/N)
    hlp_mod char(8),           # Help module
    hlp_prog char(8),          # Help program
    hlp_num smallint,          # Help number
    pick char(1),              # (Y/N) are we picking ?
    txt_default char(30),      # Key to default text (for editor)
    max_col smallint,          # maximum column size
    txt array[150] of record   # text storage
        _text char(75)
    end record

######################################################################
function textinit()
######################################################################
# This function initializes the picker/editor.
#
    # Trap fatal errors
    whenever error call error_handler

    for arr_cur = 1 to arr_cnt
        let txt[arr_cur]._text = ""
    end for
    let arr_cur = 0
    let arr_cnt = 0
    let max_col = 0
    let is_zoom = false
    let txt_default = ""
    let hlp_mod = "lib_scr"
    let hlp_prog= "textzm"
    let hlp_num = 1
    let pick = "Y"
    let is_open = false
    let reset_cnt = 123
end function
# textinit()

#######################################################################
function textput(_text)
#######################################################################
# This function puts the passed text into the next element of the
# txt array.
#
    define
        _text char(70),   # passed text
        n smallint       # generic number

    # Don't allow recursion
    if is_open = "Y" then return end if

    # Initialize the array
    if reset_cnt = 123
    then else
        call textinit()
    end if

    # Incriment array pointer
    let arr_cnt = arr_cnt + 1
    let arr_cur = arr_cnt
    if arr_cnt > 150
    then
        call lib_error("lib_scr","textzm",1,"")
        return
    end if

    # Set text if passed in scratch
    if _text = "(see scratch)" then let _text = scratch end if

    # Bump up max_col if necessary
    let n = length(_text)
    if n > max_col then let max_col = n end if

    # Set text into array
    let txt[arr_cur]._text = _text

end function
# textput()

#######################################################################
function textzoom()
#######################################################################
# This function tells the editor that you wish to display the
# (Zoom) message onto the menu line.
#
    let is_zoom = true
end function
# textzoom()

#######################################################################
function textsel()
# returning the number of elements selected.
#######################################################################
# This function executes the selection statement stored in scratch,
# and returns the first column into the txt[] array.
#
    define
        n smallint       # generic number

    # Don't allow recursion
    if is_open = "Y" then return 0 end if

    # Initialize the array
    if reset_cnt = 123
    then else
        call textinit()
    end if

    # Prepare the select statement
    let status = 0
    whenever error continue
    prepare sel_stmt from scratch
    if status = 0
    then
        declare sel_curs cursor for sel_stmt
        if status = 0
        then
            open sel_curs
        end if
    end if
    whenever error call error_handler

    # Check for completion of prepare/declare/open
    if sqlca.sqlcode != 0 or status != 0
    then
        call lib_error("lib_scr","textzm",2,"(see scratch)")
        return 0
    end if

    let n = 0
    foreach sel_curs into scratch
        call textput("(see scratch)")
        let n = n + 1
        # Return upon too many rows selected
        if arr_cnt > 150
        then
            let arr_cnt = 150
            return -1
        end if
    end foreach

    # Return the number of elements selected
    return n

end function
# textsel()

#######################################################################
function textget()
# returning false if the array limit is passed, true otherwise
# returning the text stored in scratch
#######################################################################
# This function returns the specified element from the txt[] array.
#

    # Don't allow recursion
    if is_open = "Y" then return false end if

    # Return false if outside range
    if arr_cur > arr_cnt or arr_cur < 1 then return false end if

    # Return the text
    let scratch = txt[arr_cur]._text
    let arr_cur = arr_cur + 1
    return true

end function
# textget()

#######################################################################
function textshow(is_update)
# returning false if [DEL] is pressed, true otherwise.
#######################################################################
# This function opens a window and displays the text stored in
# the txt[] array.  It puts the user into 'display array' or
# 'input array' depending on is_update false/true.
#
    define
        is_update smallint, # (boolean) is this window for update?
        num_cols smallint,  # text window size (horizontal)
        scr_lin smallint,   # current screen line
        did_zoom smallint,  # Did the user press ZOOM?
        old_cols smallint,  # Storage for max_col
        old_default char(30),# Storage for previous default
        n smallint          # generic counter

    # Notify the put routine to start again
    let reset_cnt = true

    # Open the correct window
    case
      when max_col < 21
        open window twin at  6,50 with form "lib_tex2" attribute(border)
      when max_col < 31
        open window twin at  6,40 with form "lib_tex3" attribute(border)
      when max_col < 41
        open window twin at  6,30 with form "lib_tex4" attribute(border)
      when max_col < 51
        open window twin at  6,20 with form "lib_tex5" attribute(border)
      when max_col < 61
        open window twin at  6,10 with form "lib_tex6" attribute(border)
      otherwise
        open window twin at  6, 3 with form "lib_tex7" attribute(border)
    end case

    call window_size()
    call menu_line()
    if is_zoom then call lib_message("zoom_on") end if
    let is_zoom = false   # Reset for next call
    let did_zoom = false  # User didn't press zoom (yet)
    let is_open = "Y"  # Prevent recursive calls
    let num_cols = get_scrlib("num_cols")

    # Display the heading (centered)
    let n = (num_cols / 2) - (length(heading) / 2) + .99
    if n < 1 then let n = 1 end if
    display heading at 4, n

    # Enter display or input array
    let int_flag = 0
    call set_count(arr_cnt)
    if is_update
    then
        call lib_message("update")
        input array txt without defaults from s_text.*
          before field txt
            let arr_cur = arr_curr()
            let scr_lin = scr_line()
            display txt[arr_cur]._text to s_text[scr_lin].txt
              attribute(reverse)
          after field txt
            display txt[arr_cur]._text to s_text[scr_lin].txt

          # Event trapping logic
          on key (control-b) let hotkey = 2  goto event1
          on key (control-e) let hotkey = 5  goto event1
          on key (control-f) let hotkey = 6  goto event1
          on key (control-g) let hotkey = 7  goto event1
          on key (control-i) let hotkey = 9  goto event1
          on key (control-n) let hotkey = 14 goto event1
          on key (control-o) let hotkey = 15 goto event1
          on key (control-p) let hotkey = 16 goto event1
          on key (control-t) let hotkey = 20 goto event1
          on key (control-u) let hotkey = 21 goto event1
          on key (control-v) let hotkey = 22 goto event1
          on key (control-w) let hotkey = 23 goto event1
          on key (control-y) let hotkey = 25 goto event1
          on key (control-z) let hotkey = 26 goto event1
          on key (f5)  let hotkey = 105 goto event1
          on key (f6)  let hotkey = 106 goto event1
          on key (f7)  let hotkey = 107 goto event1
          on key (f8)  let hotkey = 108 goto event1
          on key (f9)  let hotkey = 109 goto event1
          on key (f10) let hotkey = 110 goto event1
          on key (f11) let hotkey = 111 goto event1
          on key (f12) let hotkey = 112 goto event1
          on key (f13) let hotkey = 113 goto event1
          on key (f14) let hotkey = 114 goto event1
          on key (f15) let hotkey = 115 goto event1
          on key (f16) let hotkey = 116 goto event1
          on key (f34) let hotkey = 134 goto event1
          on key (f35) let hotkey = 135 goto event1
          on key (f36) let hotkey = 136 goto event1

          label event1:
            call hot_local("todo")
            call hot_local("feature")
            call hot_local("ack")
            call hot_local("info")
            call hot_local("hot")
            call hot_local("navigate")
            call hot_local("help")
            call hot_key("textedit")
            case
              when scr_funct = "todo"
                or scr_funct = "feature"
                or scr_funct = "ack"
                or scr_funct = "info"
                or scr_funct = "hot"
                or scr_funct = "navigage"
                let int_flag = -123
                exit input
              when scr_funct = "help"
                call lib_help(hlp_mod, hlp_prog, hlp_num)
              when scr_funct = "zoom"
                let did_zoom = true
                exit input
              when scr_funct = "accept" exit input
              when scr_funct = "cancel"
                let int_flag = -1
                exit input
            end case
        end input
        # Remove blank lines at end of array
        for arr_cnt = arr_count() to 1 step -1
            if txt[arr_cnt]._text is not null then exit for end if
        end for
    else

        if pick = "Y"
          then call lib_message("choose")
          else call lib_message("textview")
        end if
        # Display the number of rows selected
        let scratch = "(", arr_cnt using "<<&", " items)"
        call lib_message("scr_bottom")
        display array txt to s_text.* attribute(red)

          # Event trapping logic
          on key (control-b) let hotkey = 2  goto event2
          on key (control-e) let hotkey = 5  goto event2
          on key (control-f) let hotkey = 6  goto event2
          on key (control-g) let hotkey = 7  goto event2
          on key (control-i) let hotkey = 9  goto event2
          on key (control-n) let hotkey = 14 goto event2
          on key (control-o) let hotkey = 15 goto event2
          on key (control-p) let hotkey = 16 goto event2
          on key (control-t) let hotkey = 20 goto event2
          on key (control-u) let hotkey = 21 goto event2
          on key (control-v) let hotkey = 22 goto event2
          on key (control-w) let hotkey = 23 goto event2
          on key (control-y) let hotkey = 25 goto event2
          on key (control-z) let hotkey = 26 goto event2
          on key (f5)  let hotkey = 105 goto event2
          on key (f6)  let hotkey = 106 goto event2
          on key (f7)  let hotkey = 107 goto event2
          on key (f8)  let hotkey = 108 goto event2
          on key (f9)  let hotkey = 109 goto event2
          on key (f10) let hotkey = 110 goto event2
          on key (f11) let hotkey = 111 goto event2
          on key (f12) let hotkey = 112 goto event2
          on key (f13) let hotkey = 113 goto event2
          on key (f14) let hotkey = 114 goto event2
          on key (f15) let hotkey = 115 goto event2
          on key (f16) let hotkey = 116 goto event2
          on key (f34) let hotkey = 134 goto event2
          on key (control-m, f35) let hotkey = 135 goto event2
          on key (f36) let hotkey = 136 goto event2

          label event2:
            call hot_local("todo")
            call hot_local("feature")
            call hot_local("ack")
            call hot_local("info")
            call hot_local("hot")
            call hot_local("navigate")
            call hot_local("help")
            call hot_key("textpick")
            case
              when scr_funct = "todo"
                or scr_funct = "feature"
                or scr_funct = "ack"
                or scr_funct = "info"
                or scr_funct = "hot"
                or scr_funct = "navigate"
                let int_flag = -123
                exit display
              when scr_funct = "help"
                call lib_help(hlp_mod, hlp_prog, hlp_num)
              when scr_funct = "zoom"
                let did_zoom = true
                exit display
              when scr_funct = "accept" exit display
              when scr_funct = "cancel"
                let int_flag = -1
                exit display
            end case
        end display
    end if

    if int_flag = -123 then else let scr_funct = "" end if

    # Close and resize the window
    close window twin
    let is_open = "N"
    call window_size()

    # Any 'textpick' type events selected?
    case
      when scr_funct = "todo" call fg_todo() let int_flag = -1
      when scr_funct = "feature" call fg_feature() let int_flag = -1
      when scr_funct = "ack" call fg_ack() let int_flag = -1
      when scr_funct = "info" call fg_info() let int_flag = -1
      when scr_funct = "hot" call hot_browse() let int_flag = -1
      when scr_funct = "navigate" call fg_navigate() let int_flag = -1
      when scr_funct = "info" call fg_info() let int_flag = -1
      when did_zoom and is_update = false
        # Send the zoom down to the picker's calling function
        let arr_cur = arr_curr()
        let scratch = "zoom", txt[arr_cur]._text
        return false
      when did_zoom and is_update and txt_default is not null
        # If zooming in the editor, edit the default text
        # Remember the key for the delete/insert
        let old_default = txt_default
        let old_cols = max_col
        call textdflt()
        let max_col = old_cols
        let scratch = heading clipped, " (Default)"
        let heading = scratch
        if textshow(true) = true
        then
            whenever error continue
            delete from stxtxtdd where txt_key = old_default
            let n = 1
            while textget()
                insert into stxtxtdd values(old_default, n, scratch)
                let n = n + 1
            end while
            whenever error call error_handler
        end if
        # Lines were overwritten, so exit as if user pressed DEL
        let int_flag = -1
    end case

    let scr_funct = ""

    # Determine array element chosen
    if int_flag != 0
    then
        # Interrupt pressed
        let int_flag = 0
        let arr_cnt  = 0
        let arr_cur  = 0
        let scratch  = ""
        return false
    else
        # Determine the array element to return
        let arr_cur = arr_curr()
        if arr_cur > arr_cnt then let arr_cur = arr_cnt end if
        if arr_cur > 0
        then
            let scratch = txt[arr_cur]._text
            let arr_cur = 1    # reset arr_cur for textget()
        else
            let scratch = ""
            let arr_cur = 0    # reset arr_cur for textget()
        end if
        return true
    end if

end function
# textshow()

#######################################################################
function textpick(head)
# returning the line number of the picked element or 0 if [DEL] pressed.
# also returning picked data in scratch if selected.
# returns 0 and scratch[1,4]="zoom" and scratch[5,74] zoom line text
# (if you need the line # for zoom, call arr_curr())
#######################################################################
# This function places the user into display array on the txt[] array.
# It is designed as a 'picker' to select from a list.
#
    define
        head char(80)       # screen heading (passed)

    # Don't allow recursion
    if is_open = "Y"
    then
        call lib_error("lib_scr","recursiv",1,
          "Window:  Generic text picker/editor")
        return false
    end if

    # Assign the static 'heading' variable
    let heading = head

    # Bump up max_col if heading is bigger than max_col
    if max_col < length(heading)
      then let max_col = length(heading)
    end if

    # Call textshow (no update - view only)
    if textshow(false) = false
      then return 0
      else return arr_curr()
    end if

end function
# textpick()

#######################################################################
function textview(head)
#######################################################################
# This function places the user into display array on the txt[] array.
# It is exactly like textpick() except the message says Info vs. Pick.
# It is designed to be used to display information, not to pick.
#
    define
        head char(80)       # screen heading (passed)

    # We're not picking, we're showing
    let pick = "N"
    if textpick(head) then end if

end function
# textview()

#######################################################################
function textedit(head, win_size)
# returning true if text edited, false if [DEL] pressed.
#######################################################################
# This function places the user into input array on the txt[] array.
# It is designed as a generic 4GL text editor.
# A screen heading is passed, and a window size.  The window size
# determines which screen is opened.  Screens are available from 20
# characters to 74 characters (in even incriments of 10 except 74)
# If the user presses [DEL], then 0 is returned.
#
    define
        head char(80),     # screen heading (passed)
        win_size smallint  # window size (horizontal)

    # Don't allow recursion
    if is_open = "Y"
    then
        call lib_error("lib_scr","recursiv",1,
          "Window:  Generic text picker/editor")
        return false
    end if

    # Fill the editor with the default text if there are no lines
    if arr_cnt = 0 and txt_default is not null
      then call textdflt()
    end if

    # Assign the static variables
    let heading = head
    let max_col = win_size

    # Call textshow (for update)
    return textshow(true)

end function
# textedit()

#######################################################################
function texthelp(h_mod, h_prog, h_num)
#######################################################################
# This function identifies the help module/program/number that is
# used when the user requests help.
#
    define
        h_mod char(8),           # Help module
        h_prog char(8),          # Help program
        h_num smallint           # Help number

    # Don't allow recursion
    if is_open = "Y" then return end if

    # Assign to 'static' variables
    let hlp_mod = h_mod
    let hlp_prog = h_prog
    let hlp_num = h_num

end function
# texthelp()

#######################################################################
function texthlp(h_num)
#######################################################################
# This function identifies the help number that is used when the user
# requests help.  The module/program are gotten from progid.
#
    define
        h_num smallint           # Help number

    # Don't allow recursion
    if is_open = "Y" then return end if

    # Assign to 'static' variables
    call get_mod() returning hlp_mod, hlp_prog
    let hlp_num = h_num

end function
# texthlp()

#######################################################################
function textdefault(default_key)
#######################################################################
# This function defines the unique key to use to place default text
# into the editor.
# The default text will be placed into the editor if there are 0 lines
# of text to be edited upon entry into the editor.
# If there is a default key, the (Zoom) message will appear, and if
# the user presses ZOOM during text entry, they will be able to edit
# their default text.
#
    define
        default_key char(30)  # Default lines key

    # Don't allow recursion
    if is_open = "Y" then return end if

    # Assign the default key
    let txt_default = default_key
    let is_zoom = true

end function
# textdefault()

#######################################################################
function textdflt()
#######################################################################
# This function fills the array with the default lines
#
    define
        quote char(1)  # Storage for the quote character

    let quote = "\""
    let scratch = "select dflt_text, line_no from stxtxtdd ",
      "where txt_key = ", quote, txt_default clipped, quote,
      " order by line_no"
    if textsel() then end if

end function
# textdflt()
