########################################################################
# Copyright (C) 1991
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../standard.4gs/var_arg.4gl  1.3  Delta: 1/9/91
########################################################################

globals "globals.4gl"

define
    max_len smallint,    # Maximum length of all arguments
    arg_cur smallint,    # Current argument number
    arg_num smallint,    # Number of arguments
    arg_new smallint,    # (boolean) new call to vararg?
    arg_start smallint,  # starting position of current argument
    arg_ptr array[100] of smallint  # pointers to argument ending char

######################################################################
function put_vararg(arg_value)
######################################################################
# This is the "put" portion of the vararg family of functions.
# The vararg functions allow a variable number of arguments to be
# passed between functions.  
# The vararg functions use the 'scratch' variable, so no manipulation
# of that variable can be done between calls to put and get_vararg.
# The limits of the vararg routines are:
# - maximum 100 arguments
# - no argument may exceed 100 characters in length
# - all arguments total must not exceed 2048 characters
#
    define
        arg_value char(100),  # value passed
        arg_end    smallint,  # ending position of argument
        arg_length smallint   # length of value

    # Trap fatal errors
    whenever error call error_handler

    # Reset pointer and number of elements upon new call to put_vararg
    if arg_new = 123
    then else
        let max_len = 0
        let arg_cur = 0
        let arg_num = 0
        let arg_start = 1
        let arg_new = 123
        let scratch = " "  # make sure scratch isn't null
    end if

    # Incriment argument pointers
    let arg_cur = arg_cur + 1
    if arg_cur > 100
      then call lib_error("lib_scr","vararg",1,"Max = 100")
    end if
    let arg_num = arg_cur
    let arg_length = length(arg_value)
    let arg_end = arg_start + arg_length - 1
    let arg_ptr[arg_cur] = arg_end

    # Place the data into scratch (if not null)
    if arg_length > 0
      then let scratch[arg_start,arg_end] = arg_value[1,arg_length]
    end if

    # Bump up the maximum length if this string is longer
    if arg_length > max_len then let max_len = arg_length end if

    # Set arg_start for next invocation
    let arg_start = arg_end + 1
end function
# put_vararg()

######################################################################
function max_vararg()
# returning length of the longest string in the stack.
######################################################################
# A call to this function is necessary if you wish to know the length
# of the longest string in the vararg stack.
#
    return max_len
end function
# num_vararg()

######################################################################
function num_vararg()
# returning the number of elements stored in vararg()
######################################################################
# A call to this function is necessary if you wish to know how many
# arguments are stored in scratch.
#
    return arg_num
end function
# num_vararg()

######################################################################
function get_vararg()
# returning the argument as a string
######################################################################
# This is the "get" portion of the vararg family of functions.
# This function returns the argument
#
    define
        arg_end    smallint,  # ending position of argument
        new_start  smallint   # new starting position

    # Is this the first call to get_vararg() after put_vararg()?
    if arg_new = 123
    then
        let arg_new = 0
        let arg_cur = 0
        let arg_start = 1
    end if
        
    # Incriment the current argument
    let arg_cur = arg_cur + 1
    if arg_cur > arg_num then return "" end if
    let arg_end = arg_ptr[arg_cur]

    # Set arg_start for next invocation
    let new_start = arg_start
    let arg_start = arg_end + 1

    # Return the data  (or null)
    if arg_end >= new_start
      then return scratch[new_start,arg_end]
      else return ""
    end if
end function
# get_vararg()

######################################################################
function getx_vararg(arg_no)
# returning the argument as a string
######################################################################
# This is another form of the  "get_vararg" function.  It allows you to
# get an exact argument from the vararg stack.  It doesn't affect the
# continuity of the get_vararg function, so you can use them both in a
# get_vararg session.
#
    define
        arg_no     smallint,  # argument being requested
        arg_start  smallint,  # starting position of argument
        arg_end    smallint   # ending position of argument

    # Is this the first call to getx_vararg() after put_vararg()?
    if arg_new = 123
      then let arg_new = 0
    end if
        
    # Is the argument outside the limits?
    if arg_no < 0 or arg_no > arg_num then return "" end if

    # Set the start/end pointers
    if arg_no = 1
      then let arg_start = 1
      else let arg_start = arg_ptr[arg_no - 1] + 1
    end if
    let arg_end = arg_ptr[arg_no]

    # Return the data  (or null)
    if arg_end >= arg_start
      then return scratch[arg_start,arg_end]
      else return ""
    end if
end function
# getx_vararg()
